package com.zyh.system.service.impl;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.zyh.common.constants.RabbitMQQueueNameConstants;
import com.zyh.common.domain.Camera;
import com.zyh.common.domain.CompreFaceApiResponse;
import com.zyh.common.domain.dto.CompreFaceApiResponseDto;
import com.zyh.common.result.Result;
import com.zyh.common.util.ComprefaceUtils;
import com.zyh.system.mapper.CameraMapper;
import com.zyh.system.mapper.ComprefaceMapper;
import com.zyh.system.service.ComprefaceService;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.UUID;

@Service
public class ComprefaceServicempl implements ComprefaceService {


    @Autowired
    private CameraMapper cameraMapper;
    @Autowired
    private ComprefaceMapper comprefaceMapper;


    @Autowired
    RabbitTemplate rabbitTemplate;

    @Override
    public Result<CompreFaceApiResponse.Result> compareWithSingleFace(MultipartFile targetImage, String cameraId) throws IOException {
        byte[] bytes = targetImage.getBytes();

        Result<CompreFaceApiResponse.Result> result = ComprefaceUtils.recognizeFace(bytes);
        CompreFaceApiResponse.Result data = result.getData();
        //查询设备信息
        Camera camera = cameraMapper.selectCamera(cameraId);
        data.setCamera(camera);


        ObjectMapper objectMapper = new ObjectMapper();
        String jsonString = objectMapper.writeValueAsString(data);

        rabbitTemplate.convertAndSend(RabbitMQQueueNameConstants.SEND_SMS_QUEUE_NAME, jsonString, message -> {

            message.getMessageProperties().setMessageId(UUID.randomUUID().toString().replaceAll("-", ""));
            return message;
        });


        rabbitTemplate.convertAndSend(RabbitMQQueueNameConstants.FACE_CONTROLS, bytes, message -> {

            message.getMessageProperties().setMessageId(UUID.randomUUID().toString().replaceAll("-", ""));
            message.getMessageProperties().setHeader("cameraId", cameraId); // 设置消息头中的相机ID
            return message;
        });

        return Result.success(data);
    }


}
