package com.zyh.system.mq;

import com.baidu.aip.util.Base64Util;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.rabbitmq.client.Channel;
import com.zyh.common.constants.RabbitMQQueueNameConstants;
import com.zyh.common.domain.Alarm;
import com.zyh.common.domain.CompreFaceApiResponse;
import com.zyh.common.face.FaceMatchUtil;
import com.zyh.common.result.Result;
import com.zyh.common.util.ComprefaceUtils;
import com.zyh.system.service.AlarmService;
import com.zyh.system.service.FaceControlsService;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.List;

@Component
@Log4j2
public class FaceControlsConsumer {
    @Autowired
    private AlarmService alarmService;
    @Autowired
    private FaceControlsService faceControlsService;
    @Autowired
    RedisTemplate<String, String> redisTemplate;

    @RabbitListener(queuesToDeclare = {@Queue(RabbitMQQueueNameConstants.FACE_CONTROLS)})
    public void SendCodeMq(byte[] bytes, Message message, Channel channel) throws JsonProcessingException {
        long l = System.currentTimeMillis();
        log.info("短信消费者队列开始消费...，消费内容为：{}", bytes);
        String messageId = message.getMessageProperties().getMessageId();
        String cameraId = message.getMessageProperties().getHeaders().get("cameraId").toString();

        try {
            Long count = redisTemplate.opsForSet().add(RabbitMQQueueNameConstants.FACE_CONTROLS, messageId);
            if (count == 1) {
                float num = 0;
                Result<CompreFaceApiResponse.Result> result = ComprefaceUtils.recognizeFace(bytes);
                CompreFaceApiResponse.Result data = result.getData();
                String encode = Base64Util.encode(bytes);
                List<String> pics = faceControlsService.getFaceControlsPic();
                for (String pic : pics) {
                    float v = FaceMatchUtil.matchImages(encode, pic);
                    if (v > num) { // 如果当前v的值大于num，则更新num的值为v
                        num = v;
                    }
                }
                if (num > 0.95) {


                    alarmService.addAlarm(data,cameraId);

                }
                log.info("短信消费者队列消费成功，耗时：{}毫秒！", System.currentTimeMillis()-l);
            }else{
                log.info("短信消费者队列重复消费，消费内容为：{}", bytes);
            }

        } catch (Exception e) {
            log.info("短信消费者队列消费失败，异常内容为：{}", e.getMessage());
            redisTemplate.opsForSet().remove(RabbitMQQueueNameConstants.FACE_CONTROLS, messageId);

            try {
                channel.basicReject(message.getMessageProperties().getDeliveryTag(),true);
            } catch (Exception ex) {
                log.info("短信消费者队列回退失败，异常内容为：{}", ex.getMessage());
            }
        }
    }

}
